@@ -1,5 +1,8 @@
 package Data::Dump::Color;
 
+our $DATE = '2014-10-29'; # DATE
+our $VERSION = '0.23'; # VERSION
+
 use 5.010001;
 use strict;
 use vars qw(@EXPORT @EXPORT_OK $VERSION $DEBUG);
@@ -10,42 +13,74 @@ require Exporter;
 @EXPORT = qw(dd ddx);
 @EXPORT_OK = qw(dump pp dumpf quote);
 
-our $VERSION = '0.16'; # VERSION
-$DEBUG = 0;
+$DEBUG = $ENV{DEBUG};
 
 use overload ();
 use vars qw(%seen %refcnt @fixup @cfixup %require $TRY_BASE64 @FILTERS $INDENT);
-use vars qw(%COLORS $COLOR $INDEX);
+use vars qw(%COLOR_THEMES %COLORS $COLOR $COLOR_THEME $COLOR_DEPTH $INDEX $LENTHRESHOLD);
 
 use Term::ANSIColor;
 require Win32::Console::ANSI if $^O =~ /Win/;
-use Scalar::Util qw(looks_like_number);
+use Scalar::Util::LooksLikeNumber qw(looks_like_number);
 
 $TRY_BASE64 = 50 unless defined $TRY_BASE64;
 $INDENT = "  " unless defined $INDENT;
 $INDEX = 1 unless defined $INDEX;
+$LENTHRESHOLD = 500 unless defined $LENTHRESHOLD;
 
-%COLORS = (
-    Regexp  => 'yellow',
-    undef   => 'bright_red',
-    number  => 'bright_blue', # floats can have different color
-    float   => 'cyan',
-    string  => 'bright_yellow',
-    object  => 'bright_green',
-    glob    => 'bright_cyan',
-    key     => 'magenta',
-    comment => 'green',
-    keyword => 'blue',
-    symbol  => 'cyan',
-    linum   => 'black on_white', # file:line number
+%COLOR_THEMES = (
+    default16 => {
+        colors => {
+            Regexp  => 'yellow',
+            undef   => 'bright_red',
+            number  => 'bright_blue', # floats can have different color
+            float   => 'cyan',
+            string  => 'bright_yellow',
+            object  => 'bright_green',
+            glob    => 'bright_cyan',
+            key     => 'magenta',
+            comment => 'green',
+            keyword => 'blue',
+            symbol  => 'cyan',
+            linum   => 'black on_white', # file:line number
+        },
+    },
+    default256 => {
+        color_depth => 256,
+        colors => {
+            Regexp  => 135,
+            undef   => 124,
+            number  => 27,
+            float   => 51,
+            string  => 226,
+            object  => 10,
+            glob    => 10,
+            key     => 202,
+            comment => 34,
+            keyword => 21,
+            symbol  => 51,
+            linum   => 10,
+        },
+    },
 );
+
+$COLOR_THEME = ($ENV{TERM} // "") =~ /256/ ? 'default256' : 'default16';
+$COLOR_DEPTH = $COLOR_THEMES{$COLOR_THEME}{color_depth} // 16;
+%COLORS      = %{ $COLOR_THEMES{$COLOR_THEME}{colors} };
+
 my $_colreset = color('reset');
 sub _col {
     my ($col, $str) = @_;
+    my $colval = $COLORS{$col};
     if ($COLOR // $ENV{COLOR} // (-t STDOUT)) {
-        color($COLORS{$col}) . $str . $_colreset;
+        #say "D:col=$col, COLOR_DEPTH=$COLOR_DEPTH";
+        if ($COLOR_DEPTH >= 256 && $colval =~ /^\d+$/) {
+            return "\e[38;5;${colval}m" . $str . $_colreset;
+        } else {
+            return color($colval) . $str . $_colreset;
+        }
     } else {
-        $str;
+        return $str;
     }
 }
 
@@ -104,6 +139,7 @@ sub dump
                                [0],
                                [map {defined($_->[1]) ? $_->[1] : "\$".$_->[0]} @dump ],
                                [map {defined($_->[1]) ? $_->[1] : "\$".$_->[0]} @cdump],
+                               \@_,
                            );
     $out  .= $f;
     $cout .= $cf;
@@ -355,12 +391,12 @@ sub _dump
             push @cvals, $cd;
 	    $i++;
 	}
-	my ($f, $cf) = format_list(1, $tied, [scalar(@$idx)], \@vals, \@cvals);
+	my ($f, $cf) = format_list(1, $tied, [scalar(@$idx)], \@vals, \@cvals, $rval);
         $out  = "[$f]";
         $cout = "[$cf]";
     }
     elsif ($type eq "HASH") {
-	my(@keys, @vals, @cvals);
+	my(@keys, @vals, @cvals, @origk, @origv);
 	my $tied = tied_str(tied(%$rval));
 
 	# statistics to determine variation in key lengths
@@ -395,6 +431,7 @@ sub _dump
         my @lenvlastline;
         for my $key (@orig_keys) {
 	    my $val = \$rval->{$key};  # capture value before we modify $key
+	    push(@origk, $key);
 	    $key = quote($key) if $quote;
 	    $kstat_max = length($key) if length($key) > $kstat_max;
 	    $kstat_sum += length($key);
@@ -404,9 +441,10 @@ sub _dump
             my ($v, $cv) = _dump($$val, $name, [@$idx, ["{$key}","{"._col(string=>$key)."}"]], $tied, $pclass, $pidx);
 	    push(@vals ,  $v);
 	    push(@cvals, $cv);
+	    push(@origv, $$val);
 
             my ($vlastline) = $v =~ /(.*)\z/;
-            #say "DEBUG: vlastline=<$vlastline>";
+            #say "DEBUG: v=<$v>, vlastline=<$vlastline>" if $DEBUG;
             my $lenvlastline = length($vlastline);
             push @lenvlastline, $lenvlastline;
 	}
@@ -436,6 +474,9 @@ sub _dump
 		    push(@vals, sprintf("%.2f (%d/%.1f/%.1f)",
 					$stddev / $kstat_max,
 					$kstat_max, $avg, $stddev));
+		    push(@cvals, sprintf("%.2f (%d/%.1f/%.1f)",
+                                         $stddev / $kstat_max,
+                                         $kstat_max, $avg, $stddev));
 		}
 	    }
 	}
@@ -459,17 +500,23 @@ sub _dump
 	    my $key = shift(@keys);
 	    my $val  = shift @vals;
 	    my $cval = shift @cvals;
+	    my $origk = shift @origk;
+	    my $origv = shift @origv;
             my $lenvlastline = shift @lenvlastline;
-	    my $vpad = $INDENT . (" " x ($klen_pad ? $klen_pad + 4 : 0));
+	    my $vmultiline = length($val) > $lenvlastline;
+            my $vpad = $INDENT . (" " x ($klen_pad ? $klen_pad + 4 : 0));
 	    $val  =~ s/\n/\n$vpad/gm;
 	    $cval =~ s/\n/\n$vpad/gm;
 	    my $kpad = $nl ? $INDENT : " ";
 	    $key .= " " x ($klen_pad - length($key)) if $nl;
-            my $cpad = " " x ($maxkvlen - length($key) - $lenvlastline);
-            #say "DEBUG: key=<$key>, val=<$val>, lenvlastline=<$lenvlastline>, cpad=<$cpad>";
-            my $idxcomment = sprintf "# %s{%${idxwidth}i}", "." x @$idx, $i;
-	    $out  .= "$kpad$key => $val," . ($nl && $INDEX ? " $cpad$idxcomment" : "") . $nl;
-	    $cout .= $kpad._col(key=>$key)." => $cval,".($nl && $INDEX ? " $cpad"._col(comment => $idxcomment) : "") . $nl;
+            my $cpad = " " x ($maxkvlen - ($vmultiline ? -6+length($vpad) : length($key)) - $lenvlastline);
+            #say "DEBUG: key=<$key>, vpad=<$vpad>, val=<$val>, lenvlastline=<$lenvlastline>, cpad=<$cpad>" if $DEBUG;
+            my $visaid = "";
+            $visaid .= sprintf("%s{%${idxwidth}i}", "." x @$idx, $i) if $INDEX;
+            $visaid .= " klen=".length($origk) if length($origk) >= $LENTHRESHOLD;
+            $visaid .= " vlen=".length($origv) if length($origv) >= $LENTHRESHOLD;
+	    $out  .= "$kpad$key => $val," . ($nl && length($visaid) ? " $cpad# $visaid" : "") . $nl;
+	    $cout .= $kpad._col(key=>$key)." => $cval,".($nl && length($visaid) ? " $cpad"._col(comment => "# $visaid") : "") . $nl;
             $i++;
 	}
 	$out  =~ s/,$/ / unless $nl;
@@ -566,6 +613,7 @@ sub format_list
     my $indent_lim = $paren ? 0 : 1;
     my @vals  = @{ shift(@_) };
     my @cvals = @{ shift(@_) };
+    my @orig  = @{ shift(@_) };
 
     if (@vals > 3) {
 	# can we use range operator to shorten the list?
@@ -592,6 +640,7 @@ sub format_list
 	    if ($j - $i > 3) {
 		splice(@vals , $i, $j - $i, "$vals[$i] .. $vals[$j-1]");
 		splice(@cvals, $i, $j - $i, "$cvals[$i] .. $cvals[$j-1]");
+		splice(@orig , $i, $j - $i, [@orig[$i..$j-1]]);
 	    }
 	    $i++;
 	}
@@ -618,9 +667,11 @@ sub format_list
         for my $i (0..$#elem) {
             my ($vlastline) = $elem[$i] =~ /(.*)\z/;
             my $cpad = " " x ($maxvlen - length($vlastline));
-            my $idxcomment = sprintf "# %s[%${idxwidth}i]", "." x $extra->[0], $i;
-            push @res , $elem[ $i], ",", ($INDEX ? " $cpad$idxcomment" : ""), "\n";
-            push @cres, $celem[$i], ",", ($INDEX ? " $cpad"._col(comment => $idxcomment) : ""), "\n";
+            my $visaid = "";
+            $visaid .= sprintf("%s[%${idxwidth}i]", "." x $extra->[0], $i) if $INDEX;
+            $visaid .= " len=".length($orig[$i]) if length($orig[$i]) >= $LENTHRESHOLD;
+            push @res , $elem[ $i], ",", (length($visaid) ? " $cpad# $visaid" : ""), "\n";
+            push @cres, $celem[$i], ",", (length($visaid) ? " $cpad"._col(comment => "# $visaid") : ""), "\n";
         }
         return (join("", @res), join("", @cres));
     } else {
@@ -708,13 +759,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Data::Dump::Color - Like Data::Dump, but with color
 
 =head1 VERSION
 
-version 0.16
+This document describes version 0.23 of Data::Dump::Color (from Perl distribution Data-Dump-Color), released on 2014-10-29.
 
 =head1 SYNOPSIS
 
@@ -725,15 +778,18 @@ Use it like you would Data::Dump, e.g.:
 =head1 DESCRIPTION
 
 This module aims to be a drop-in replacement for L<Data::Dump>. It adds colors
-to dumps. For more information, see Data::Dump. This documentation explains
-what's different between this module and Data::Dump.
+to dumps. It also adds various visual aids in the comments, e.g. array/hash
+index, depth indicator, and so on.
+
+For more information, see Data::Dump. This documentation explains what's
+different between this module and Data::Dump.
 
 =for Pod::Coverage .+
 
 =head1 RESULTS
 
-By default Data::Dump::Color shows array indexes or hash pair sequence in
-comments for visual aid, e.g.:
+By default Data::Dump::Color shows array index or hash pair sequence in comments
+for visual aid, e.g.:
 
  [
    "this",      # [0]
@@ -798,6 +854,11 @@ Define colors.
 
 Whether to add array/hash index visual aid.
 
+=item $LENTHRESHOLD => int (default: 500)
+
+Add string length visual aid for hash key/hash value/array element if length
+is at least this value.
+
 =back
 
 =head1 ENVIRONMENT
@@ -832,13 +893,29 @@ L<SHARYANTO::Role::ColorTheme>.
 
 L<Data::Dump>, L<JSON::Color>, L<YAML::Tiny::Color>
 
+=head1 HOMEPAGE
+
+Please visit the project's homepage at L<https://metacpan.org/release/Data-Dump-Color>.
+
+=head1 SOURCE
+
+Source repository is at L<https://github.com/perlancar/perl-Data-Dump-Color>.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Data-Dump-Color>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =head1 AUTHOR
 
-Steven Haryanto <stevenharyanto@gmail.com>
+perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Steven Haryanto.
+This software is copyright (c) 2014 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
